@@ -1,8 +1,11 @@
 from neurograd import xp
+from neurograd import xp
 from typing import TYPE_CHECKING, Union, Tuple, Sequence, Literal
 from numpy.typing import ArrayLike
 if TYPE_CHECKING:
     from neurograd.tensor import Tensor
+if TYPE_CHECKING:
+    from neurograd.tensor import Tensor
 
 
 
@@ -60,11 +63,13 @@ def conv2d(input: Union["Tensor", xp.ndarray], filters: Union["Tensor", xp.ndarr
     if not depthwise:
         slides = slider(input)  # (N, C, out_H, out_W, F_H, F_W)
         filters = filters # (F_N, C, F_H, F_W)
-        output = ng.einsum("ncpqhw,fchw->nfpq", slides, filters, use_opt_einsum=True) # (N, F_N, out_H, out_W) 
+        # Use opt_einsum for standard convolutions (benchmark winner)
+        output = ng.einsum("ncpqhw,fchw->nfpq", slides, filters, backend="opt_einsum") # (N, F_N, out_H, out_W) 
     else:
         slides = slider(input)  # (N, C, out_H, out_W, F_H, F_W)
         filters = filters # (C, F_H, F_W)
-        output = ng.einsum('ncpqhw,chw->ncpq', slides, filters, use_opt_einsum=True) # (N, C, out_H, out_W)
+        # Use native xp einsum for depthwise convolutions (benchmark winner)
+        output = ng.einsum('ncpqhw,chw->ncpq', slides, filters, backend="xp") # (N, C, out_H, out_W)
     return output
 
 
